//=============================================================================
// MrFu_AufUndAbRunden.js
//=============================================================================
/*:de
* @plugindesc v1.2 Ermöglicht mathematisches Runden und Berechnungen mit Dezimalzahlen.
*
* @param Alle Variablen
* @type boolean
* @desc True wählen, falls dieses Plugin auf alle Variablen zugreifen soll. Andernfalls Parameter 'Variablen' verwenden.
* @default false
* @on Alle Variablen
* @off Nur ausgewählte Variablen
*
* @param Variablen
* @type []
* @desc Variablen IDs: Pro Zeile entweder eine ID oder einen Bereich eintragen. Bsp: 25-68
*
* @author Mr. Fu
*
* @help 
*
* Dieses Plugin dient zur mathematischen Rundung von Variablen auf ganze Zahlen
* (also auf- und abrunden) und zum Rechnen mit Dezimalzahlen. Zuerst werden die 
* Funktionen erklärt, danach die Funktionsweise des Plugins anhand von einigen
* Beispielen.
*
* Die Funktionen können mit der Schaltfläche "Skript..." aber nicht mit der
* Schaltfläche "Plug-in-Befehl..." aufgerufen werden und beziehen sich nur auf
* die Variablen, die in den Parametern festgelegt wurden:
*
* $runden.dezimal()       // Diesen Befehl vor die Berechnung der Variablen im
*                            Spiel setzen. Die jeweiligen Variablen sind nun
*                            als Dezimalzahl formatiert.
*
* $runden.start()         // Diesen Befehl ausführen sobald die nötigen Be-
*                            rechnungen durchgeführt wurden. Die jeweiligen
*                            Variablen werden nun gerundet und wieder zu
*                            Ganzzahlen umformatiert.
*
* $runden.nicht_dezimal() // Dieser Befehl formatiert die Variablen wieder
*                            in ganze Zahlen um.
*
*-----------------------------------------------------------------------------
*
* Funktionsweise und Beispiele:
*
* Bevor Berechnungen mit den Variablen durchgeführt werden sollen, muss die
* Funktion '$runden.dezimal()' ausgeführt werden. Ab jetzt kann man mit
* Dezimalzahlen rechnen. Allerdings werden die Zahlen auch als Dezimalzahlen
* ausgegeben, z. B. in einem Textfenster. Selbst wenn die Zahl keine
* Nachkommastellen hat. (Bsp: 4.0)
* Sobald alle nötigen Berechnungen durchgeführt wurden kann die nächste Funktion
* ausgeführt werden. Wenn man die Zahlen nun endgültig runden möchte, nutzt man
* die Funktion '$runden.start()'. Sie rundet nun alle Variablen mathematisch
* (X.0 - X.4 wird abgerundet, ab X.5 aufgerundet). Wollte man hingegen mit den
* Dezimalzahlen rechnen und nicht runden, dann muss anstatt '$runden.start()'
* die Funktion '$runden.nicht_dezimal()' nutzen. Beide Funktionen wandeln die
* Dezimalzahlen wieder in Ganzzahlen um, sodass sie wieder wie gewohnt in der
* Textverarbeitung ausgegeben werden können.
*
* ============================================================================
* Terms of Use
* ============================================================================
* Dieses Plugin darf frei genutzt werden, wenn Credits gegeben werden.
*
* Kontakt: Mr. Fu auf www.rpgmaker-mv.de
* ============================================================================
* Changelog
* ============================================================================
* Version 1.2: 24.01.2020
* - Fehler behoben, der das Plugin abstürzen ließ, wenn der Parameter 'Alle
*   Variablen' auf 'true' gesetzt wurde und die Funktion '$runden.start()' bzw.
*   '$runden.nicht_dezimal()' ausgeführt wurde.
*
* Version 1.1: 18.01.2020
* - Code überarbeitet
* - Parameter typisiert
* - Möglichkeit hinzugefügt um Bereiche für aufeinander folgende Variablen
*   festzulegen
* - Beschreibung überarbeitet
*
* Version 1.0: 28.02.2018
* - Release!
*/
(function() { // Zieht die Parameter
function parseParameters(params) {
	var obj;
	try {
		obj = JsonEx.parse(params && typeof params === 'object' ? JsonEx.stringify(params) : params);
	} catch (e) {
				return params;
	} // end of try
	if (obj && typeof obj === 'object') {
		Object.keys(obj).forEach(function (key) {
			obj[key] = parseParameters(obj[key]); 
			// If the parameter has no value, meaning it's an empty string, just set it to null
			if (obj[key] === '') {
				obj[key] = null;
			} // end of if
		});
	} // end of if
	return obj;
} // end of parseParameters
var params = parseParameters(PluginManager.parameters('MrFu_AufUndAbRunden'));
var alleVar = params['Alle Variablen'];
var varArray = params['Variablen'];
var varBereich = [];
var anzahl = 0;
var durchlauf = 0;
function Runden() { // Definiert die Klasse Runden
	this.dezimal = function() { // Funktion um Dezimalzahlen zu erhalten
		if (alleVar) {
			Game_Variables.prototype.setValue = function(variableId, value) {
				if (variableId > 0 && variableId < $dataSystem.variables.length) {
	                this._data[variableId] = value;
					this.onChange();
				} // end of if
			} // end of setValue
		} else {
			anzahl = varArray.length - 1;
			Game_Variables.prototype.setValue = function(variableId, value) {
				if (variableId > 0 && variableId < $dataSystem.variables.length) {
					durchlauf = 0
					while (durchlauf <= anzahl) {
						if (variableId === varArray[durchlauf]) {
							break
						} else {
							if (typeof value === 'number') {
								value = Math.floor(value);
							} // end of if
						} // end of if
						durchlauf ++;
					} // end of while
					this._data[variableId] = value;
					this.onChange();
				} // end of if
			} // end of setValue
		} // end of if
	}; // end of dezimal
	this.start = function() { // Funktion um die Variablen zu runden
		durchlauf = 0
		Game_Variables.prototype.roundValue = function(variableId) {
			this._data[variableId] = Math.round(this._data[variableId]);
		}; // end of roundValue
		if (alleVar === true) {
			for (var i = 1; i < $gameVariables._data.length; i++) {
				if (!isNaN($gameVariables._data[i])) {
					$gameVariables.roundValue(i);	
				} // end of if
			} // end of for
		} else{
			while (durchlauf <= anzahl) {
				$gameVariables.roundValue(varArray[durchlauf]);
				durchlauf ++;
			} // end of while
		} // end of if
		$runden.nicht_dezimal();
	}; // end of start	
	this.nicht_dezimal = function() { // Funktion um zukünftig keine Dezimalzahlen zu erhalten
		Game_Variables.prototype.setValue = function(variableId, value) {
			if (variableId > 0 && variableId < $dataSystem.variables.length) {
				if (typeof value === 'number') {
					value = Math.floor(value);
				} // end of if
				this._data[variableId] = value;
				this.onChange();
			} // end of if
		}; // end of setValue
		durchlauf = 0;
		Game_Variables.prototype.roundValue = function(variableId) {
			this._data[variableId] = Math.floor(this._data[variableId]);
		}; // end of roundValue
		if (alleVar === true) {
			for (var i = 1; i < $gameVariables._data.length; i++) {
				if (!isNaN($gameVariables._data[i])) {
					$gameVariables.roundValue(i);	
				} // end of if
			} // end of for
		} else{
			while (durchlauf <= anzahl) {
				$gameVariables.roundValue(varArray[durchlauf]);
				durchlauf ++;
			} // end of while
		} // end of if
	}; // end of nicht_dezimal
} // end of Runden
if (alleVar !== true) { // Erstellt ein Array falls nicht alle Variablen genutzt werden sollen.
	durchlauf = 0;
	while (durchlauf < varArray.length) { // Prüft ob ungültige Werte in dem Array vorhanden sind.
		if (varArray[durchlauf] <= 0 || isNaN(varArray[durchlauf]) || varArray[durchlauf] > 5000) {
			if (isNaN(varArray[durchlauf])) {
				if (varArray[durchlauf].search(/-/) === -1) {
					alert("Meldung im Plugin 'MrFu_AufUndAbRunden':\n\nEin Eintrag im Parameter 'Variablen' ist keine Zahl und kein gültiger Bereich. Diese ID wird nicht berücksichtigt.");
					varArray.splice(durchlauf, 1);
				} else {
					varBereich.push(varArray[durchlauf]);
					varArray.splice(durchlauf, 1);
				} // end of if
			} else {
				alert("Meldung im Plugin 'MrFu_AufUndAbRunden':\n\nEin Eintrag im Parameter 'Variablen' hat einen ungültigen Wert (eine Zahl über 5000 bzw. unter 1). Diese ID wird nicht berücksichtigt.");
				varArray.splice(durchlauf, 1);
			} // end of if
		} else {
			durchlauf++;
		} // end of if
	} // end of while
	if (varBereich.length !== 0) {
		while (isNaN(varBereich[0])) {
			varBereich.push(Number(varBereich[0].substring(0, varBereich[0].search(/-/))));
			varBereich.push(Number(varBereich[0].substring(varBereich[0].search(/-/) + 1)));
			if (isNaN(varBereich[varBereich.length - 2]) || isNaN(varBereich[varBereich.length - 1])) {
				alert("Meldung im Plugin 'MrFu_AufUndAbRunden':\n\nEin Eintrag im Parameter 'Variablen' ist kein gültiger Bereich. Diese ID wird nicht berücksichtigt.");
				varBereich.pop();
				varBereich.pop();
			} // end of if
			varBereich.shift();
			if (varBereich.length === 0) {
				break
			} // end of if
		} // end of while
		while (varBereich.length !== 0) {
			for (var i = varBereich[0]; i <= varBereich[1]; i++) {
				varArray.push(i);
			} // end of for
			varBereich.shift();
			varBereich.shift();
		} // end of while
	} // end of if
} // end of if
$runden = new Runden;
})();